/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL
v2. You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#include "rm_scan.h"
#include "record/rm_defs.h"
#include "rm_file_handle.h"

/**
 * @brief 初始化file_handle和rid
 * @param file_handle
 */
RmScan::RmScan(const RmFileHandle *file_handle) : file_handle_(file_handle) {
    // Todo:
    // 初始化file_handle和rid（指向第一个存放了记录的位置）
    auto page_no = RM_NO_PAGE;
    auto slot_no = -1;
    auto max_record = file_handle_->file_hdr_.num_records_per_page;
    for (page_no = 1; page_no < file_handle_->file_hdr_.num_pages; page_no++) {
        auto page_handle = file_handle->fetch_page_handle(page_no);
        slot_no = Bitmap::first_bit(true, page_handle.bitmap, max_record);
        // file_handle_->buffer_pool_manager_->unpin_page(
        //     page_handle.page->get_page_id(), false);
        if (slot_no < max_record) {
            break;
        }
        slot_no = -1;
    }
    assert(slot_no != -1);
    rid_ = Rid{page_no, slot_no};
}

/**
 * @brief 找到文件中下一个存放了记录的位置
 */
void RmScan::next() {
    // Todo:
    // 找到文件中下一个存放了记录的非空闲位置，用rid_来指向这个位置
    int pos = -1;
    int max_record =
        file_handle_->file_hdr_.num_records_per_page; // 每页最大记录数
    RmPageHandle page_handle = file_handle_->fetch_page_handle(rid_.page_no);
    while (true) {
        pos =
            Bitmap::next_bit(true, page_handle.bitmap, max_record,
                             rid_.slot_no); // 在当前页面从当前位置寻找下一位置
        rid_.slot_no = pos;
        if (pos != max_record || is_end()) { // 找到记录位置或者搜索完全部页面
            return;
        }
        page_handle = file_handle_->fetch_page_handle(++(rid_.page_no));
        rid_.slot_no = -1;
    }
}

/**
 * @brief ​ 判断是否到达文件末尾
 */
bool RmScan::is_end() const {
    // Todo: 修改返回值
    int max_pages = file_handle_->file_hdr_.num_pages;
    int max_record =
        file_handle_->file_hdr_.num_records_per_page; // 每页最大记录数
    return rid_.page_no >= max_pages - 1 && rid_.slot_no >= max_record;
}

/**
 * @brief RmScan内部存放的rid
 */
Rid RmScan::rid() const { return rid_; }